(function() {
	var tad = window.tad = {};
	
	/**
	 * This method allows the ad to confirm a basic feature set before display.
	 * The version number corresponds with the API specification version and not a vendor's SDK version.
	 * @param none
	 * @returns {String} the API specification that this container is certified against
	 */
	tad.getVersion = function() {
		return "2.1";
	};
	
	/**
	 * The method will send an event to log server.
	 * @param {String}  an event code for user action.
	 * @returns none
	 */
	tad.sendEventCode = function(action) {
		TadController.sendEventCode(action);
	};
	
	tad.downloadApplication = function(downloadInfo) {
		TadController.downloadApplication(stringify(downloadInfo));
	};
	
	tad.runApplication = function(packageInfo, downloadInfo) {
		TadController.runApplication(stringify(packageInfo), stringify(downloadInfo));
	};
	
	/**
	 * Use this method to Check uri can open or not
	 * @param {String} uri
	 * @returns {boolean} 
	 * @remark Included in tad version: 2.0
	 */
	tad.canOpenUri = function(uri) {
		return TadController.canOpenUri(uri);
	};
	
	/**
	 * Use this method to save picture  
	 * @param {String} uri
	 * @returns none
	 * @remark Included in tad version: 2.0
	 */
	tad.storePicture = function(url, title, description) {
		TadController.storePicture(url, title, description);
	};
	
	/**
	 * Get media's set clientId
	 * @param none
	 * @returns {String} clientId
	 * @remark Included in tad version: 2.0
	 */
	tad.getClientId = function() {
		return TadController.getClientId();
	};
	
	/**
	 * Get deviceId
	 * @param none
	 * @returns {String} deviceId
	 * @remark Included in tad version: 2.0
	 */
	tad.getDeviceId = function() {
		return TadController.getDeviceId();
	};
	
	/**
	 * Get SDK version.
	 * @param none
	 * @returns {String} SDK version
	 * @remark Included in tad version: 2.0
	 */
	tad.getSdkVersion = function() {
		return TadController.getSdkVersion();
	};
	
	/**
	 * Get phoneNumber
	 * @param none
	 * @returns {String} phoneNumber
	 * @remark Included in tad version: 2.0
	 */
	tad.getPhoneNumber = function() {
		return TadController.getPhoneNumber();
	};

	/**
	 * Get EncryptedPhoneNumber
	 * @param none
	 * @returns {String} EncryptedPhoneNumber
	 * @remark Included in tad version: 2.0
	 */
	tad.getEncryptedPhoneNumber = function() {
		return TadController.getEncryptedPhoneNumber();
	};
	
	/**
	 * Get AgreementState
	 * @param none
	 * @returns {Integer} AgreementState
	 * @remark Included in tad version: 2.0
	 */
	tad.getAgreementState = function() {
		return TadController.getAgreementState();
	};
	
	/**
	 * Get AppVersionCode
	 * @param {String} packageName
	 * @returns {String} appVersionCode
	 * @remark Included in tad version: 2.0
	 */
	tad.getAppVersionCode = function(packageName) {
		return TadController.getAppVersionCode(packageName);
	};

	/**
	 * Get AppVersionName
	 * @param {String} packageName
	 * @returns {String} appVersionName
	 * @remark Included in tad version: 2.0
	 */
	tad.getAppVersionName = function(packageName) {
		return TadController.getAppVersionName(packageName);
	};
	
	/*
	tad.runApplications = function(schemeInfoList) {
		TadController.runApplications(stringify(schemeInfoList));
	};
	*/
	
	tad.playVideo = function(url) {
		TadController.playVideo(url);
	};
	
	tad.tstoreRichReward = function(url) {
		TadController.tstoreRichReward(url);
	};
	
	stringify = function(args) {
		if (typeof JSON === "undefined") {
			var s = "";
			var len = args.length;
			var i;
			if (typeof len == "undefined") {
				return stringifyArg(args);
			}
			for (i = 0; i < args.length; i++) {
				if (i > 0) {
					s = s + ",";
				}
				s = s + stringifyArg(args[i]);
			}
			s = s + "]";
			return s;
		} else {
			return JSON.stringify(args);
		}
	};

	stringifyArg = function(arg) {
		var s, type, start, name, nameType, a;
		type = typeof arg;
		s = "";
		if ((type === "number") || (type === "boolean")) {
			s = s + args;
		} else if (arg instanceof Array) {
			s = s + "[" + arg + "]";
		} else if (arg instanceof Object) {
			start = true;
			s = s + '{';
			for (name in arg) {
				if (arg[name] !== null) {
					if (!start) {
						s = s + ',';
					}
					s = s + '"' + name + '":';
					nameType = typeof arg[name];
					if ((nameType === "number") || (nameType === "boolean")) {
						s = s + arg[name];
					} else if ((typeof arg[name]) === 'function') {
						// don't copy the functions
						s = s + '""';
					} else if (arg[name] instanceof Object) {
						s = s + this.stringify(args[i][name]);
					} else {
						s = s + '"' + arg[name] + '"';
					}
					start = false;
				}
			}
			s = s + '}';
		} else {
			a = arg.replace(/\\/g, '\\\\');
			a = a.replace(/"/g, '\\"');
			s = s + '"' + a + '"';
		}
		
		return s;
	};
})();
