(function() {
	var mraid = window.mraid = window.ormma = {};

	/**
	 * Constants
	 */
	var STATES = mraid.STATES = {
		LOADING		: 'loading',
		DEFAULT		: 'default',
		RESIZED		: 'resized',
		EXPANDED	: 'expanded',
		HIDDEN		: 'hidden'
	};

	var EVENTS = mraid.EVENTS = {
		ERROR				: 'error',
		READY				: 'ready',
		SIZECHANGE			: 'sizeChange',
		STATECHANGE			: 'stateChange',
		VIEWABLECHANGE		: 'viewableChange'
	};

	var FEATURES = mraid.FEATURES = {
		SMS			: 'sms',
		TEL			: 'tel',
		CALENDAR	: 'calendar',
		PICTURE		: 'storePicture',
		INLINEVIDEO	: 'inlineVideo'
	};

	// PRIVATE PROPERTIES (sdk controlled) //////////////////////////////////////////////////////

	var currentState = STATES.LOADING;

	var currentExpandProperties = null;

	var currentOrientationProperties = null;

	var currentResizeProperties = null;

	var currentSupports = {
		'sms'			: true,
		'tel'			: true,
		'calendar'		: true,
		'storePicture'	: true,
		'inlinevideo'	: true
	};

	// PRIVATE PROPERTIES (internal) //////////////////////////////////////////////////////

	var changeHandlers = {
		onExpandProperties : function(val) {
			currentExpandProperties = val;
		},
		onOrientationProperties : function(val) {
			currentOrientationProperties = val;
		},
		onReady : function(val) {
			broadcastEvent(EVENTS.READY);
		},
		onResizeProperties : function(val) {
			currentResizeProperties = val;
		},
		onSize : function(val) {
			broadcastEvent(EVENTS.SIZECHANGE, val.width, val.height);
		},
		onState : function(val) {
			currentState = val;
			broadcastEvent(EVENTS.STATECHANGE, currentState);
		},
		onSupports : function(val) {
			currentSupports = val;
		},
		onViewable : function(val) {
			broadcastEvent(EVENTS.VIEWABLECHANGE, val);
		}
	};

	var listeners = {};

	var EventListeners = function(event) {
		this.event = event;
		this.count = 0;
		var listeners = {};

		this.add = function(func) {
			var id = String(func);
			if (!listeners[id]) {
				listeners[id] = func;
				this.count++;
			}
		};
		this.remove = function(func) {
			var id = String(func);
			if (listeners[id]) {
				listeners[id] = null;
				delete listeners[id];
				this.count--;
				return true;
			} else {
				return false;
			}
		};
		this.removeAll = function() {
			for (var id in listeners) {
				this.remove(listeners[id]);
			}
		};
		this.broadcast = function(args) {
			for (var id in listeners) {
				listeners[id].apply({}, args);
			}
		};
		this.toString = function() {
			var out = [ event, ':' ];
			for (var id in listeners) {
				out.push('|', id, '|');
			}
			return out.join('');
		};
	};

	var clone = function(obj) {
		var f = function() {
		};
		f.prototype = obj;
		return new f();
	};
	
	var parse = function(args) {
		if (typeof JSON === "undefined") {
			return eval('(' + args	+ ')');
		} else {
			return JSON.parse(args);
		}
	}

	/*var stringify_simple = function(obj) {
		if (typeof obj == 'object') {
			var out = [];
			if (obj.push) {
				for (var p in obj) {
					out.push(obj[p]);
				}
				return '[' + out.join(',') + ']';
			} else {
				for (var p in obj) {
					out.push('\'' + p + '\':' + obj[p]);
				}
				return '{' + out.join(',') + '}';
			}
		} else {
			return String(obj);
		}
	};*/

	stringify = function(args) {
		if (typeof JSON === "undefined") {
			var s = "";
			var len = args.length;
			var i;
			if (typeof len == "undefined") {
				return stringifyArg(args);
			}
			for (i = 0; i < args.length; i++) {
				if (i > 0) {
					s = s + ",";
				}
				s = s + stringifyArg(args[i]);
			}
			s = s + "]";
			return s;
		} else {
			return JSON.stringify(args);
		}
	};

	stringifyArg = function(arg) {
		var s, type, start, name, nameType, a;
		type = typeof arg;
		s = "";
		if ((type === "number") || (type === "boolean")) {
			s = s + args;
		} else if (arg instanceof Array) {
			s = s + "[" + arg + "]";
		} else if (arg instanceof Object) {
			start = true;
			s = s + '{';
			for (name in arg) {
				if (arg[name] !== null) {
					if (!start) {
						s = s + ',';
					}
					s = s + '"' + name + '":';
					nameType = typeof arg[name];
					if ((nameType === "number") || (nameType === "boolean")) {
						s = s + arg[name];
					} else if ((typeof arg[name]) === 'function') {
						// don't copy the functions
						s = s + '""';
					} else if (arg[name] instanceof Object) {
						s = s + this.stringify(args[i][name]);
					} else {
						s = s + '"' + arg[name] + '"';
					}
					start = false;
				}
			}
			s = s + '}';
		} else {
			a = arg.replace(/\\/g, '\\\\');
			a = a.replace(/"/g, '\\"');
			s = s + '"' + a + '"';
		}
		
		return s;
	};

	var contains = function(value, array) {
		for (var i in array) {
			if (array[i] == value) {
				return true;
			}
		}
		return false;
	};

	var broadcastEvent = function() {
		var args = new Array(arguments.length);
		for (var i = 0; i < arguments.length; i++) {
			args[i] = arguments[i];
		}
		var event = args.shift();
		try {
			if (listeners[event]) {
				listeners[event].broadcast(args);
			}
		} catch (e) {
		}
	};

	var trim = function(s) {
		var l = 0;
		var r = s.length - 1;
		while (l < s.length && s[l] == ' ') {
			l++;
		}
		while (r > l && s[r] == ' ') {
			r -= 1;
		}
		return s.substring(l, r + 1);
	};

	/*
	 * LEVEL 1 ////////////////////////////////////////////////////////////////////
	 */

	/**
	 * Use this method to subscribe a specific handler method to a specific event.
	 * @param {String} name of event to listen for
	 * @param {Function} function name (or anonymous function) to execute
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.addEventListener = function(event, listener) {
		if (!event || !listener) {
			broadcastEvent(EVENTS.ERROR, 'Both event and listener are required.', 'addEventListener');
		} else if (!contains(event, EVENTS)) {
			broadcastEvent(EVENTS.ERROR, 'Unknown event: ' + event, 'addEventListener');
		} else {
			if (!listeners[event]) {
				listeners[event] = new EventListeners(event);
			}
			listeners[event].add(listener);
		}
	};

	/**
	 * The close method will cause the ad webview to downgrade its state.
	 * @param none
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.close = function() {
		try {
			MRAIDDisplayControllerBridge.close();
		} catch (e) {
			MRAIDUtilityControllerBridge.showAlert("close: " + e);
		}
	};

	/**
	 * Use this method to create a new event in the devices calendar.
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 2
	 */
	mraid.createCalendarEvent = function(val) {
		var description = val.description;
		var location = val.location;
		var summary = val.summary;
		var start = new Date(val.start);
		var end = new Date(val.end);

		if (!currentSupports[FEATURES.CALENDAR]) {
			broadcastEvent(EVENTS.ERROR, 'Method not supported by this client.', 'createCalendarEvent');
		} else {
			try {
				MRAIDUtilityControllerBridge.createCalendarEvent(description, location, summary, start.getTime().toString(), end.getTime().toString());
			} catch (e) {
				MRAIDUtilityControllerBridge.showAlert("createCalendarEvent: " + e);
			}
		}
	};

	/**
	 * The expand method will cause an existing Web View (for one-part creatives)
	 *  or a new Web View (for two-part creatives) to open at the highest z-order
	 *  in the view hierarchy.
	 * @param {String}  The optional url for the document to be displayed in a new overlay view.
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.expand = function(url) {
		try {
			MRAIDDisplayControllerBridge.expand(url, stringify(currentExpandProperties), stringify(currentOrientationProperties));
		} catch (e) {
			MRAIDUtilityControllerBridge.showAlert("expand: " + e);
		}
	};

	/**
	 * Use this parameter to get the location and size of the current ad view.
	 * @param none
	 * @returns {Object} the default position {x, y, width, height}
	 * @remark Included in MRAID version: 2
	 * @remark level: 1
	 */
	mraid.getCurrentPosition = function() {
		return parse(MRAIDDisplayControllerBridge.getCurrentPosition());
	};

	/**
	 * Use this parameter to get the location and size of the default ad view.
	 * @param none
	 * @returns {Object} the default position {x, y, width, height}
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 1
	 */
	mraid.getDefaultPosition = function() {
		return parse(MRAIDDisplayControllerBridge.getDefaultPosition());
	};

	/**
	 * Use this method to get the properties for expanding an ad.
	 * @param none
	 * @returns {Object} the expand properties
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.getExpandProperties = function() {
		return currentExpandProperties == null ? null : clone(parse(currentExpandProperties));
	};

	/**
	 * Use this method to return the maximum size an ad can grow to using the resize() method.
	 * @param none
	 * @returns {Object} the maximum width and height the view can grow to {width, height}
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 1
	 */
	mraid.getMaxSize = function() {
		return parse(MRAIDDisplayControllerBridge.getMaxSize());
	};

	/**
	 * Use this method to get the orientation properties for expandable ad or interstitial ad.
	 * @param none
	 * @returns {Object} the orientation properties
	 * @remark Included in MRAID version: 2
	 * @remark level: 1
	 */
	mraid.getOrientationProperties = function() {
		return currentOrientationProperties == null ? null : clone(parse(currentOrientationProperties));
	};

	/** TODO
	 * retrieve the current banner type
	 * @param none
	 * @returns {String} either "interstitial" or "inline"
	 * @remark Introduced in ORMMA version: 1.1.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.getPlacementType = function() {
		return MRAIDDisplayControllerBridge.getPlacementType();
	};

	/**
	 * Use this method to get the properties for resizing an ad.
	 * @param none
	 * @returns {Object} the resize properties
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.getResizeProperties = function() {
		return currentResizeProperties == null ? null : clone(parse(currentResizeProperties));
	};

	/**
	 * Use this method to get the current pixel width and height of the device.
	 * @param none
	 * @returns {Object} size {"width, height"}
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 2
	 */
	mraid.getScreenSize = function() {
		return parse(MRAIDDisplayControllerBridge.getScreenSize());
	};

	/**
	 * This method returns the current state of the ad container,
	 *  returning whether the ad container is in its default, fixed position,
	 *  a changed, resized position, or is in an expanded, larger position.
	 * @param none
	 * @returns {String} state "loading", "default", "resized", "expanded" or "hidden"
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.getState = function() {
		return currentState;
	};

	/**
	 * This method allows the ad to confirm a basic feature set before display.
	 * The version number corresponds with the API specification version and not a vendor's SDK version.
	 * @param none
	 * @returns {String} the API specification that this container is certified against
	 * @remark Introduced in ORMMA version: 1.1.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.getVersion = function() {
		return "2.0";
	};

	/**
	 * TODO This method returns whether the web container view showing the ad is
	 * currently on screen.
	 * 
	 * @param none
	 * @returns {Boolean} true, container is on-screen and viewable by the user;
	 *          false: container is off-screen and not viewable
	 * @remark Introduced in ORMMA version: 1.1.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.isViewable = function() {
		return MRAIDDisplayControllerBridge.isViewable();
	};

	/**
	 * Use this method to open an HTML browser to an external web
	 * page.
	 * 
	 * @param {String}
	 *            the url of the web page to open
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.open = function(url) {
		if (!url) {
			broadcastEvent(EVENTS.ERROR, 'url is required.', 'open');
		} else {
			try {
				MRAIDDisplayControllerBridge.open(url);
			} catch (e) {
				MRAIDUtilityControllerBridge.showAlert("open: " + e);
			}
		}
	};

	/**
	 * Use this method to play a video on the device.
	 * This may launch an external player, depending on the SDK implementation.
	 * @param {String} the url of the video or video stream
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.1
	 * @remark level: 2
	 */
	mraid.playVideo = function(url) {
		if (!url || typeof url != 'string') {
			broadcastEvent(EVENTS.ERROR, 'Request must specify a url', 'playVideo');
		} else {
			try {
				MRAIDDisplayControllerBridge.playVideo(url);
			} catch (e) {
				MRAIDUtilityControllerBridge.showAlert("playVideo: " + e);
			}
		}
	};

	/**
	 * Use this method to unsubscribe a specific handler method from a specific event.
	 * @param {String} name of event
	 * @param {String} name of function to be removed
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.removeEventListener = function(event, listener) {
		if (!event) {
			broadcastEvent(EVENTS.ERROR, 'Must specify an event.', 'removeEventListener');
		} else {
			if (listener
					&& (!listeners[event] || !listeners[event].remove(listener))) {
				broadcastEvent(EVENTS.ERROR,
						'Listener not currently registered for event', 'removeEventListener');
				return;
			} else if (listeners[event]) {
				listeners[event].removeAll();
			}

			if (listeners[event] && listeners[event].count == 0) {
				listeners[event] = null;
				delete listeners[event];
			}
		}
	};

	/**
	 * Use this method to resize the main ad view to the desired size.
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 1
	 */
	mraid.resize = function() {
		try {
			MRAIDDisplayControllerBridge.resize(stringify(currentResizeProperties));
		} catch (e) {
			MRAIDUtilityControllerBridge.showAlert("resize: " + e);
		}
	};

	/**
	 * Use this method to set the ad's expand properties.
	 * @param {Object} properties
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.setExpandProperties = function(properties) {
//		var fields = [ 'width', 'height', 'useCustomClose', 'isModal', 'useBackground', 'backgroundColor', 'backgroundOpacity' ];
		var fields = [ 'width', 'height', 'useCustomClose', 'isModal' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentExpandProperties[field] = properties[field];
			}
		}
	};

	/**
	 * Use this method to set the ad's orientation properties.
	 * @param {Object} properties
	 * @returns none
	 * @remark Included in MRAID version: 2
	 * @remark level: 1
	 */
	mraid.setOrientationProperties = function(properties) {
		var fields = [ 'allowOrientationChange', 'forceOrientation' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentOrientationProperties[field] = properties[field];
			}
		}

		MRAIDDisplayControllerBridge.setOrientationProperties(stringify(currentOrientationProperties));
	};

	/**
	 * Use this method to set the ad's resize properties.
	 * @param {Object} properties
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.setResizeProperties = function(properties) {
		var fields = [ 'width', 'height', 'customClosePosition', 'offsetX', 'offsetY', 'allowOffscreen' ];

		for (var f in fields) {
			var field = fields[f];
			if (properties[field] !== undefined) {
				currentResizeProperties[field] = properties[field];
			}
		}
	};

	/**
	 * This method will store the image or other media type specified by
	 * the url.
	 * 
	 * @param {String}
	 *            the url to the image or other media asset
	 * @returns none
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 2
	 */
	mraid.storePicture = function(url) {
		if (!currentSupports[FEATURES.PICTURE]) {
			broadcastEvent(EVENTS.ERROR, 'Method not supported by this client.', 'storePicture');
		} else {
			MRAIDUtilityControllerBridge.storePicture(url);
		}
	};

	/**
	 * return whether the feature is supported or not.
	 * @param {String} the url to the image or other media asset
	 * @returns {Boolean} true, the feature is supported and getter and events are available; false, the feature is not supported
	 * @remark Introduced in ORMMA version: 1.0.0
	 * @remark level: 1
	 */
	mraid.supports = function(feature) {
		if (currentSupports[feature]) {
			return true;
		} else {
			return false;
		}
	};

	/**
	 * set whether custom close use or not
	 * @param {String} the url to the image or other media asset
	 * @returns {Boolean} true ad creative supplies its own designs for the close area; false default image will be displayed for the close area
	 * @remark Introduced in ORMMA version: 1.1.0
	 * @remark Included in MRAID version: 1
	 * @remark level: 1
	 */
	mraid.useCustomClose = function(flag) {
		mraid.setExpandProperties({
			"useCustomClose" : flag
		});

		MRAIDDisplayControllerBridge.useCustomClose(flag);
	};

	// PRIVATE METHODS ////////////////////////////////////////////////////////////

	var mraidview = window.mraidview = {};

	/** The set of listeners for Bridge Events */
	var bridge_listeners = {};

	/**
	 * Called by the SDK when various state properties have changed.
	 *
	 * @returns string, "OK"
	 */
	mraidview.fireChangeEvent = function(properties) {
		var handlers = bridge_listeners['change'];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](properties);
			}
		}

		return "OK";
	};

	/**
	 * Called by the SDK when an error has occurred.
	 *
	 * @returns string, "OK"
	 */
	mraidview.fireErrorEvent = function(message, action) {
		var handlers = bridge_listeners['error'];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](message, action);
			}
		}

		return "OK";
	};

	/**
	 *
	 */
	mraidview.addEventListener = function(event, listener) {
		var handlers = bridge_listeners[event];
		if (handlers == null) {
			// no handlers defined yet, set it up
			bridge_listeners[event] = [];
			handlers = bridge_listeners[event];
		}

		// see if the listener is already present
		for (var handler in handlers) {
			if (listener == handler) {
				// listener already present, nothing to do
				return;
			}
		}

		// not present yet, go ahead and add it
		handlers.push(listener);
	};

	/**
	 * 
	 */
	mraidview.removeEventListener = function(event, listener) {
		var handlers = bridge_listeners[event];
		if (handlers != null) {
			handlers.remove(listener);
		}
	};

	/******************************************************************************
	 * 
	 * running code
	 * 
	 ******************************************************************************/
	mraidview.addEventListener('change', function(properties) {
		for (var property in properties) {
			var handler = changeHandlers[property];
			handler(properties[property]);
		}
	});

	mraidview.addEventListener('error', function(message, action) {
		broadcastEvent(EVENTS.ERROR, message, action);
	});
})();
